@@ -12,6 +12,7 @@
  *
  */
 
+#include <math.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <sys/time.h>
@@ -142,6 +143,7 @@ gf_fontmap_new (GFFontMapType type, const guchar *path)
 
 	map = g_new (GFFontMap, 1);
 	map->next = NULL;
+	map->gpversion = 0.0;
 	map->type = type;
 	map->path = g_strdup (path);
 	map->fonts = NULL;
@@ -268,8 +270,14 @@ gf_fontmap_load_file (const guchar *filename, GFFontMapType type)
 			version = xmlGetProp (root, "version");
 			if (version) {
 				if (!strcmp (version, "2.0")) {
+					xmlChar *gpversion;
+					gpversion = xmlGetProp (root, "gpversion");
 					/* We are even right version */
 					map = gf_fontmap_new (type, filename);
+					if (gpversion) {
+						map->gpversion = (gint) floor (1000.0 * atof (gpversion) + 0.f);
+						xmlFree (gpversion);
+					}
 					gf_fm_load_fonts_2_0 (map, root);
 				}
 				xmlFree (version);
