@@ -1,6 +1,7 @@
 #define __GNOME_FONT_FACE_C__
 
 #include <config.h>
+#include <stdlib.h>
 #include <string.h>
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -10,6 +11,7 @@
 #include <stdio.h>
 #include <ctype.h>
 #include <freetype/ftoutln.h>
+#include <locale.h>
 
 #include <libgnomeprint/gnome-print-i18n.h>
 #include <libgnomeprint/gnome-font.h>
@@ -525,7 +527,7 @@ gnome_font_face_get_font (const GnomeFontFace * face, gdouble size, gdouble xres
 	g_return_val_if_fail (face != NULL, NULL);
 	g_return_val_if_fail (GNOME_IS_FONT_FACE (face), NULL);
 
-	return gnome_font_face_get_font_full (face, size, NULL);
+	return gnome_font_face_get_font_full ((GnomeFontFace *) face, size, NULL);
 }
 
 GnomeFont *
@@ -535,7 +537,7 @@ gnome_font_face_get_font_default (const GnomeFontFace * face, gdouble size)
 	g_return_val_if_fail (GNOME_IS_FONT_FACE (face), NULL);
 
 	/* fixme: At least try to look like doing something (Lauris) */
-	return gnome_font_face_get_font_full (face, size, NULL);
+	return gnome_font_face_get_font_full ((GnomeFontFace *) face, size, NULL);
 }
 
 /* fixme: */
@@ -596,10 +598,12 @@ gff_load (GnomeFontFace *face)
 		g_return_val_if_fail (ft_result == FT_Err_Ok, FALSE);
 	}
 
-	if (face->entry->type == GP_FONT_ENTRY_ALIAS) {
-		entry = ((GPFontEntryAlias *) face->entry)->ref;
-	} else {
-		entry = face->entry;
+	entry = face->entry;
+
+	while (entry && entry->type == GP_FONT_ENTRY_ALIAS) entry = ((GPFontEntryAlias *) face->entry)->ref;
+	if (!entry) {
+		g_warning ("file %s: line %d: face %s: Floating alias list detected", __FILE__, __LINE__, entry->name);
+		return FALSE;
 	}
 
 	if (GFE_IS_T1_A (entry)) {
@@ -610,7 +614,7 @@ gff_load (GnomeFontFace *face)
 		if (t1->afm.name) {
 			ft_result = FT_Attach_File (ft_face, t1->afm.name);
 			if (ft_result != FT_Err_Ok) {
-				g_warning ("file %s: line %d: face %s: Cannot attach file %s", __FILE__, __LINE__, face->entry->name, t1->afm.name);
+				g_warning ("file %s: line %d: face %s: Cannot attach file %s", __FILE__, __LINE__, entry->name, t1->afm.name);
 			}
 		}
 	} else if (GFE_IS_TT (entry)) {
@@ -628,7 +632,7 @@ gff_load (GnomeFontFace *face)
 			ft_result = FT_Attach_File (ft_face, (gchar *) l->data);
 			if (ft_result != FT_Err_Ok) {
 				g_warning ("file %s: line %d: face %s: Cannot attach file %s",
-					   __FILE__, __LINE__, face->entry->name, (gchar *) l->data);
+					   __FILE__, __LINE__, entry->name, (gchar *) l->data);
 			}
 		}
 	} else {
@@ -642,7 +646,7 @@ gff_load (GnomeFontFace *face)
 
 	ft_result = FT_Select_Charmap (ft_face, ft_encoding_unicode);
 	if (ft_result != FT_Err_Ok) {
-		g_warning ("file %s: line %d: Face %s does not have unicode charmap", __FILE__, __LINE__, face->entry->name);
+		g_warning ("file %s: line %d: Face %s does not have unicode charmap", __FILE__, __LINE__, entry->name);
 	}
 
 	face->num_glyphs = ft_face->num_glyphs;
@@ -719,8 +723,7 @@ static int gfft2_move_to (FT_Vector * to, void * user)
 
 	p.x = to->x * od->t[0] + to->y * od->t[2];
 	p.y = to->x * od->t[1] + to->y * od->t[3];
-
-	if ((p.x != s->x3) || (p.y != s->y3)) {
+	if (od->end == 0 || (p.x != s->x3) || (p.y != s->y3)) {
 		od->bp[od->end].code = ART_MOVETO;
 		od->bp[od->end].x3 = to->x * od->t[0] + to->y * od->t[2];
 		od->bp[od->end].y3 = to->x * od->t[1] + to->y * od->t[3];
@@ -991,8 +994,8 @@ gff_face_from_entry (GPFontEntry *e)
 
 /* PSO (PostScriptObject stuff */
 
-static void gff_pso_ensure_buffer_t1 (GFPSObject *pso);
-static void gff_pso_ensure_buffer_tt (GFPSObject *pso);
+static void gff_pso_ensure_buffer_t1 (GFPSObject *pso, GPFontEntryT1 *t1);
+static void gff_pso_ensure_buffer_tt (GFPSObject *pso, GPFontEntryTT *tt);
 static void gff_pso_ensure_buffer_empty (GFPSObject *pso);
 static void gf_pso_sprintf (GFPSObject * pso, const gchar * format, ...);
 static void gf_pso_ensure_space (GFPSObject *pso, gint size);
@@ -1063,19 +1066,31 @@ gnome_font_face_pso_free (GFPSObject *pso)
 void
 gnome_font_face_pso_ensure_buffer (GFPSObject *pso)
 {
+	GPFontEntry *entry;
+
 	g_return_if_fail (pso != NULL);
 
-	switch (pso->face->entry->type) {
+	entry = pso->face->entry;
+
+	while (entry && (entry->type == GP_FONT_ENTRY_ALIAS)) entry = ((GPFontEntryAlias *) entry)->ref;
+	if (!entry) {
+		g_warning ("file %s: line %d: Floating alias list detected for %s",
+			   __FILE__, __LINE__, pso->face->entry->name);
+		gff_pso_ensure_buffer_empty (pso);
+		return;
+	}
+
+	switch (entry->type) {
 	case GP_FONT_ENTRY_TYPE1:
 	case GP_FONT_ENTRY_TYPE1_ALIAS:
-		gff_pso_ensure_buffer_t1 (pso);
+		gff_pso_ensure_buffer_t1 (pso, (GPFontEntryT1 *) entry);
 		break;
 	case GP_FONT_ENTRY_TRUETYPE:
-		gff_pso_ensure_buffer_tt (pso);
+		gff_pso_ensure_buffer_tt (pso, (GPFontEntryTT *) entry);
 		break;
 	default:
 		g_warning ("file %s: line %d: Unknown face entry type %s:%d",
-			   __FILE__, __LINE__, pso->face->entry->name, pso->face->entry->type);
+			   __FILE__, __LINE__, entry->name, entry->type);
 		gff_pso_ensure_buffer_empty (pso);
 		break;
 	}
@@ -1084,20 +1099,17 @@ gnome_font_face_pso_ensure_buffer (GFPSObject *pso)
 #define INT32_LSB(q) ((q)[0] + ((q)[1] << 8) + ((q)[2] << 16) + ((q)[3] << 24))
 
 static void
-gff_pso_ensure_buffer_t1 (GFPSObject *pso)
+gff_pso_ensure_buffer_t1 (GFPSObject *pso, GPFontEntryT1 *t1)
 {
-	GPFontEntryT1 *t1;
 	struct stat st;
 	guchar * fbuf;
 	gint fh;
 	const gchar *embeddedname;
 
-	t1 = (GPFontEntryT1 *) pso->face->entry;
-
-	if (pso->face->entry->type == GP_FONT_ENTRY_TYPE1) {
-		embeddedname = pso->face->entry->psname;
+	if (t1->entry.type == GP_FONT_ENTRY_TYPE1) {
+		embeddedname = t1->entry.psname;
 	} else {
-		embeddedname = ((GPFontEntryT1Alias *) pso->face->entry)->alias;
+		embeddedname = ((GPFontEntryT1Alias *) t1)->alias;
 	}
 
 	if (!GFF_LOADED (pso->face)) {
@@ -1176,6 +1188,7 @@ gff_pso_ensure_buffer_t1 (GFPSObject *pso)
 			}
 		}
 	} else {
+		gf_pso_ensure_space (pso, st.st_size);
 		memcpy (pso->buf, fbuf, st.st_size);
 		pso->buf[st.st_size] = '\0';
 		pso->length = st.st_size;
@@ -1195,9 +1208,9 @@ gff_pso_ensure_buffer_t1 (GFPSObject *pso)
 			gchar c[256];
 			FT_Error status;
 			g = (glyph < pso->face->num_glyphs) ? glyph : 0;
-			status = FT_Get_Glyph_Name (pso->face->ft_face, glyph, c, 256);
+			status = FT_Get_Glyph_Name (pso->face->ft_face, g, c, 256);
 			if (status != FT_Err_Ok) {
-				g_warning ("file %s: line %d: Glyph %d has no name in %s", __FILE__, __LINE__, glyph, t1->pfb.name);
+				g_warning ("file %s: line %d: Glyph %d has no name in %s", __FILE__, __LINE__, g, t1->pfb.name);
 #if 0
 				gff_pso_ensure_buffer_empty (pso);
 				return;
@@ -1272,9 +1285,8 @@ gff_pso_ensure_buffer_t1 (GFPSObject *pso)
  */
 
 static void
-gff_pso_ensure_buffer_tt (GFPSObject *pso)
+gff_pso_ensure_buffer_tt (GFPSObject *pso, GPFontEntryTT *tt)
 {
-	GPFontEntryTT *tt;
 	struct stat st;
 	guchar *fbuf;
 	gint fh;
@@ -1284,11 +1296,9 @@ gff_pso_ensure_buffer_tt (GFPSObject *pso)
 	const ArtDRect *bbox;
 	gint i;
 
-	tt = (GPFontEntryTT *) pso->face->entry;
-
 	/* mmap file */
 	if (stat (tt->ttf.name, &st)) {
-		g_warning ("file %s: line %d: Face: %s: Cannot load face", __FILE__, __LINE__, pso->face->entry->name);
+		g_warning ("file %s: line %d: Face: %s: Cannot load face", __FILE__, __LINE__, tt->entry.name);
 		gff_pso_ensure_buffer_empty (pso);
 		return;
 	}
@@ -1308,7 +1318,7 @@ gff_pso_ensure_buffer_tt (GFPSObject *pso)
 		return;
 	}
 
-	embeddedname = pso->face->entry->psname;
+	embeddedname = tt->entry.psname;
 
 	strings = gp_tt_split_file (fbuf, st.st_size);
 	if (strings) {
@@ -1520,92 +1530,6 @@ gf_pso_ensure_space (GFPSObject *pso, gint size)
 	}
 }
 
-
-#if 0
-static int
-read_int32_lsb (const char *p)
-{
-  const unsigned char *q = (unsigned char *)p;
-
-  return q[0] + (q[1] << 8) + (q[2] << 16) + (q[3] << 24);
-}
-
-/* this is actually the same as a pfb to pfa converter
-
- Reference: Adobe technical note 5040, "Supporting Downloadable PostScript
- Language Fonts", page 9 */
-static char *
-pfb_to_flat (const char *input, int input_size)
-{
-	const unsigned char *in = (unsigned char *)input;
-  char *flat;
-  int flat_size, flat_size_max;
-  int in_idx;
-  int length;
-  int i;
-  const char hextab[16] = "0123456789abcdef";
-
-  flat_size = 0;
-  flat_size_max = 32768;
-  flat = g_new (char, flat_size_max);
-
-  for (in_idx = 0; in_idx < input_size;)
-    {
-      if (in[in_idx] != 128)
-	{
-	  g_free (flat);
-	  return NULL;
-	}
-      switch (in[in_idx + 1])
-	{
-	case 1:
-	  length = read_int32_lsb (input + in_idx + 2);
-	  if (flat_size + length > flat_size_max)
-	    {
-	      do
-		flat_size_max <<= 1;
-	      while (flat_size + length > flat_size_max);
-	      flat = g_realloc (flat, flat_size_max);
-	    }
-	  in_idx += 6;
-	  memcpy (flat + flat_size, in + in_idx, length);
-	  flat_size += length;
-	  in_idx += length;
-	  break;
-	case 2:
-	  length = read_int32_lsb (input + in_idx + 2);
-	  if (flat_size + length * 3 > flat_size_max)
-	    {
-	      do
-		flat_size_max <<= 1;
-	      while (flat_size + length * 3 > flat_size_max);
-	      flat = g_realloc (flat, flat_size_max);
-	    }
-	  in_idx += 6;
-	  for (i = 0; i < length; i++)
-	    {
-	      flat[flat_size++] = hextab[in[in_idx] >> 4];
-	      flat[flat_size++] = hextab[in[in_idx] & 15];
-	      in_idx++;
-	      if ((i & 31) == 31 || i == length - 1)
-		flat[flat_size++] = '\n';
-	    }
-	  break;
-	case 3:
-	  /* zero terminate the returned string */
-	  if (flat_size == flat_size_max)
-	    flat = g_realloc (flat, flat_size_max <<= 1);
-	  flat[flat_size] = 0;
-	  return flat;
-	default:
-	    g_free (flat);
-	    return NULL;
-	}
-    }
-  return flat;
-}
-#endif
-
 gchar *
 gnome_font_face_get_pfa (const GnomeFontUnsized *font)
 {
@@ -1694,62 +1618,4 @@ gnome_font_face_get_pfa (const GnomeFontUnsized *font)
 #endif
 }
 
-#if 0
-static gboolean
-gnome_font_face_gt1_load (GnomeFontFace * face)
-{
-	GPFontEntryT1 * t1;
-
-	g_return_val_if_fail ((face->private->entry->type == GP_FONT_ENTRY_TYPE1) ||
-			      (face->private->entry->type == GP_FONT_ENTRY_TYPE1_ALIAS),
-			      FALSE);
-
-	t1 = (GPFontEntryT1 *) face->private->entry;
-
-	if (!face->private->loadedfont) {
-		g_return_val_if_fail (t1->pfb.name != NULL, FALSE);
-		face->private->loadedfont = gt1_load_font (t1->pfb.name, face->private->privencoding);
-		if (!face->private->loadedfont) {
-			face->private->num_glyphs = 1;
-		}
-	}
-
-	return TRUE;
-}
-#endif
-
-#if 0
-static void
-gff_fill_zero_glyph (GnomeFontFace * face)
-{
-	GFFGlyphInfo * info;
-	static ArtBpath undef[] = {{ART_MOVETO, 0.0, 0.0, 0.0, 0.0, 50.0, 50.0},
-				   {ART_LINETO, 0.0, 0.0, 0.0, 0.0, 750.0, 50.0},
-				   {ART_LINETO, 0.0, 0.0, 0.0, 0.0, 750.0, 950.0},
-				   {ART_LINETO, 0.0, 0.0, 0.0, 0.0, 50.0, 950.0},
-				   {ART_LINETO, 0.0, 0.0, 0.0, 0.0, 50.0, 50.0},
-				   {ART_MOVETO, 0.0, 0.0, 0.0, 0.0, 100.0, 100.0},
-				   {ART_LINETO, 0.0, 0.0, 0.0, 0.0, 700.0, 100.0},
-				   {ART_LINETO, 0.0, 0.0, 0.0, 0.0, 700.0, 900.0},
-				   {ART_LINETO, 0.0, 0.0, 0.0, 0.0, 100.0, 900.0},
-				   {ART_LINETO, 0.0, 0.0, 0.0, 0.0, 100.0, 100.0},
-				   {ART_END, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
-
-	info = face->private->glyphs;
-
-	info->unicode = 0;
-	info->psname = ".notdef";
-
-	info->advance.x = 800.0;
-	info->advance.y = 0.0;
-
-	info->bbox.x0 = 50.0;
-	info->bbox.y0 = 50.0;
-	info->bbox.x1 = 750.0;
-	info->bbox.y1 = 900.0;
-
-	info->bpath = undef;
-}
-#endif
-
 
