@@ -650,7 +650,7 @@ gp_ps2_set_font_private (GnomePrintContext *pc, const GnomeFont *font)
 
 	ps2 = (GnomePrintPs2 *) pc;
 
-	if (ps2->selectedfont && (ps2->selectedfont->face == font->face) && (ps2->selectedfont->currentsize = font->size)) return 0;
+	if (ps2->selectedfont && (ps2->selectedfont->face == font->face) && (ps2->selectedfont->currentsize == font->size)) return 0;
 
 	face = gnome_font_get_face (font);
 
